@@ -1,47 +1,40 @@
 package Class::Tables;
 
 use Carp;
+use Storable qw/retrieve nstore/;
 use strict;
 use warnings;
-use vars qw/$VERSION $DBH $SQL_DEBUG $PLURALIZE $SQL_QUERIES $CASCADE/;
+use vars qw/$VERSION $DBH $DB_DRIVER $SQL_DEBUG $INFLECT $SQL_QUERIES $CASCADE/;
 
-$VERSION   = "0.25_1";
-$PLURALIZE = 1;
-$CASCADE   = 1;
+$VERSION = "0.28_1";
+$INFLECT = 1;
+$CASCADE = 1;
 
 ## flyweight data
 
-my ( %CLASS, %OBJ, %TABLE_MAP );
+my ( %CLASS, %OBJ, %TABLE_MAP, $SCHEMA_CACHE );
 
 ######################
 ## public interface ##
 ######################
 
 sub import {
-    my $class = shift;
-    for (@_) {
-        if (/^no_cascade$/) {
-            $CASCADE = 0;
-            next;
-        } elsif (/^cascade$/) {
-            $CASCADE = 1;
-            next;
-        } else {
-            croak "Unknown parameter for 'use $class' : $_\n";
-        }
-    }
+    my ($class, %args) = @_;
+    
+    $CASCADE      = $args{cascade} if exists $args{cascade};
+    $INFLECT      = $args{inflect} if exists $args{inflect};
+    $SCHEMA_CACHE = $args{cache}   if exists $args{cache};
 }
 
 sub dbh {
     my ($super, $dbh) = @_;
     croak "No DBH given" unless $dbh;
-    ($DBH, %CLASS, %OBJ, %TABLE_MAP) = $dbh;
 
-    no strict qw/refs subs/;
-    
-    *PL_N = ($PLURALIZE && eval "use Lingua::EN::Inflect; 1")
-        ? Lingua::EN::Inflect::PL_N
-        : sub { $_[0] };
+    ($DBH, $DB_DRIVER, %CLASS, %OBJ, %TABLE_MAP) =
+        ($dbh, "Class::Tables::$dbh->{Driver}{Name}");
+
+    eval "use $DB_DRIVER; 1;"
+        or croak "$dbh->{Driver}{Name} is an unsupported database driver";
 
     $super->_parse_tables();
 }
@@ -53,13 +46,10 @@ sub dbh {
 sub fetch {
     my ($class, $id) = @_;
     my $id_col = $class->_id_col;
-    
-    if (exists $OBJ{$class}{$id}) {
-        $CLASS{$class}{stub_count}{$id}++;
-        return bless \$id, $class;
-    }
-    
-    return scalar $class->_stubs("where $id_col=?", $id);
+
+    return exists $OBJ{$class}{$id}
+        ? $class->_mk_obj($id)
+        : scalar $class->_get_objs("where $id_col=?", $id);
 }
 
 sub search {
@@ -77,24 +67,29 @@ sub search {
     $clause .= " order by " . $class->_order_by;
     $clause .= " limit 1" unless wantarray;
     
-    return $class->_stubs($clause, @binds);
+    return $class->_get_objs($clause, @binds);
 }
 
 sub new {
     my ($class, %params) = @_;
-    $params{id} = undef;
+    delete $params{id};
 
     my @fields = grep { exists $CLASS{$class}{accessors}{$_} } keys %params;
     my @binds  = map { UNIVERSAL::can($_, 'id') ? $_->id : $_ } @params{@fields};
     my @cols   = map { $CLASS{$class}{accessors}{$_}{col} } @fields;
 
     my $table  = $class->_table;
-    my $sql    = "insert into $table set " . join("," => map { "$_=?" } @cols);
+    my $sql    = sprintf "insert into $table (%s) values (%s)",
+                     join("," => @cols),
+                     join("," => ("?") x @cols);
                 
     sql_do($sql, @binds) or return undef;
     
-    my $id  = sql_insertid();
-    my $obj = $class->_mk_stub($id);
+#    my $id = $DBH->last_insert_id(undef, undef, $table, $class->_id_col)
+    my $id = $DB_DRIVER->insert_id($DBH, $table, $class->_id_col)
+        or die "Couldn't get last insert id";
+     
+    my $obj = $class->_mk_obj($id);
     @{ $OBJ{$class}{$id} }{@cols} = @binds;
     
     return $obj;
@@ -104,15 +99,17 @@ sub new {
 ## inherited object methods ##
 ##############################
 
-sub id { ${ $_[0] }  }
+sub id { ${ $_[0] } }
 
 sub DESTROY {
     my $self  = shift;
     my $class = ref $self;
     my $id    = $self->id;
 
-    delete $OBJ{$class}{$id}
-        unless --$CLASS{$class}{stub_count}{$id};
+    if (0 == --$CLASS{$class}{obj_count}{$id}) {
+        delete $OBJ{$class}{$id};
+        delete $CLASS{$class}{obj_count}{$id};
+    }
 }
 
 sub AUTOLOAD {
@@ -133,6 +130,9 @@ sub field {
     my $class  = ref $self;
     my $table  = $class->_table;
     my $id_col = $class->_id_col;
+    
+    return keys %{ $CLASS{$class}{accessors} }
+        unless defined $field;
 
     croak qq{Can't locate accessor "$field" via package "$class"}
         unless exists $CLASS{$class}{accessors}{$field};
@@ -142,14 +142,14 @@ sub field {
     my $col    = $CLASS{$class}{accessors}{$field}{col};
 
     return $TABLE_MAP{$ref}->search( $col => $id, @_ )
-        if $type eq 'indirect';
+        if $type eq '1-to-n';
 
     ## lazy-load columns now
     $OBJ{$class}{$id}{$col} =
             sql_do("select $col from $table where $id_col=?", $id)
         if not exists $OBJ{$class}{$id}{$col};
 
-    if ( $type eq 'direct' ) {
+    if ( $type eq '1-to-1' ) {
         if (@_) {
             my $ref_id = UNIVERSAL::can($_[0], 'id') ? $_[0]->id : $_[0];
             
@@ -181,7 +181,7 @@ sub delete {
     my $id_col = $class->_id_col;
 
     if ($CASCADE) {    
-        my @cascade = grep { $CLASS{$class}{accessors}{$_}{type} eq 'indirect' }
+        my @cascade = grep { $CLASS{$class}{accessors}{$_}{type} eq '1-to-n' }
                       keys %{ $CLASS{$class}{accessors} };
         
         for my $accessor (@cascade) {
@@ -222,14 +222,14 @@ sub dump {
 
     my %h = map {
         my $type   = $CLASS{$class}{accessors}{$_}{type};
-        my @result = $self->$_ unless $type eq 'alias';
+        my @result = $self->$_;
         my %values;
         
-        if ($type eq 'indirect') {
+        if ($type eq '1-to-n') {
         
             $values{$_} = [ map { $_->dump(@ignore) } @result ];
             
-        } elsif ($type eq 'direct') {
+        } elsif ($type eq '1-to-1') {
             if ($result[0]) {
                 my $r = $result[0]->dump(@ignore);
                 my $prefix = $_;
@@ -263,82 +263,105 @@ sub _id_col    { $CLASS{ $_[0] }{id_col}; }
 sub _accessors { keys %{ $CLASS{ $_[0] }{accessors} }; }
 sub _order_by  { $CLASS{ $_[0] }{order_by} ||= $_[0]->_id_col; }
 
-sub _mk_stub {
+sub _mk_obj {
     my ($class, $id) = @_;
-    $CLASS{$class}{stub_count}{$id}++;
+    
+    $CLASS{$class}{obj_count}{$id}++;
     return bless \$id, $class;
 }
 
-my $MAX_ROWS = 5_000;
-
-sub _stubs {
+sub _get_objs {
     my $class  = shift;
     my $clause = shift;
-    my $id_col = $class->_id_col;
     my $table  = $class->_table;
-    my @stubs;
-    my @cols   = ($id_col, $class->_load_cols);
-    
+    my @cols   = ($class->_id_col, $class->_load_cols);
+
+    my @objs;
     my $sql    = sprintf "select %s from $table $clause", join "," => @cols;
     my $q      = sql_query($sql, @_);
     
     $q->execute(@_);
 
-    while (my $rows = $q->fetchrow_arrayref) {
-        my $id = $rows->[0];
-        push @stubs, $class->_mk_stub($id);
-        @{ $OBJ{$class}{$id} }{@cols} = @$rows;
+    while (my $row = $q->fetchrow_arrayref) {
+        my $id = $row->[0];
+        push @objs, $class->_mk_obj($id);
+        @{ $OBJ{$class}{$id} }{@cols} = @$row;
     }
 
-#    for (@{ $q->fetchall_arrayref }) {
-#        my $id = $_->[0];
-#        push @stubs, $class->_mk_stub($id);
-#        @{ $OBJ{$class}{$id} }{@cols} = @$_;
-#    }
-
-#    while (my $rows = $q->fetchall_arrayref(undef, $MAX_ROWS)) {
-#        for (@$rows) {
-#            my $id = $_->[0];
-#            push @stubs, $class->_mk_stub($id);
-#            
-#            @{ $OBJ{$class}{$id} }{@cols} = @$_;
-#        }
-#    }
-
     $q->finish;
     
-    return wantarray ? @stubs : $stubs[0];
+    return wantarray ? @objs : $objs[0];
 }
 
-
 ##################
 ## private subs ##
 ##################
 
 sub _parse_tables {
     my $super = shift;
-    my @tables;
+
+    my ($CACHE, $signature);
     
-    my $q = sql_query("show tables")
-        or die "Error listing tables";
-    while ( my ($table) = $q->fetchrow_array ) {
-        my $class             = _table_to_package_name($table);
+    if ($SCHEMA_CACHE) {
+        $signature = join "\x0" =>
+                         $DBH->{Name}, $DBH->{Driver}{Name}, $INFLECT;
+                     
+        $CACHE = eval { retrieve $SCHEMA_CACHE };
+        
+        if (exists $CACHE->{$signature}) {
+            %CLASS     = %{ $CACHE->{$signature}{CLASS} };
+            %TABLE_MAP = %{ $CACHE->{$signature}{TABLE_MAP} };
+            return;
+        }
+    }
+
+    {
+        my %memoize;
+    
+        no strict qw/refs subs/;
+        *plural = ($INFLECT && eval "use Lingua::EN::Inflect; 1")
+            ? sub { $memoize{$_[0]} ||= Lingua::EN::Inflect::PL_N(@_) }
+            : sub { $_[0] . "s" };
+    }
+
+    my %map = %{ $DB_DRIVER->map_tables($DBH) };
+
+    for my $table (keys %map) {
+        my $class = _table_to_package_name($table);
+        
+        croak "Tables '$table' and '$CLASS{$class}{table}' both become "
+            . "the '$class' class"
+            if exists $CLASS{$class};
+        
         $TABLE_MAP{$table}    = $class;
         $CLASS{$class}{table} = $table;
         
         $super->_generate_package($class);
-        push @tables, $table;
     }
-    $q->finish;
 
-    for my $table (@tables) {
+    for my $table (keys %map) {
         my $class = $TABLE_MAP{$table};
 
-        $q = sql_query("describe $table")
-            or die "Error describing table $table";
-        while ( my $hr = $q->fetchrow_hashref ) {
-            my $col = $hr->{Field};
+        for my $col ( @{ $map{$table}{col_order} } ) {
+        
+            my $col_type = $map{$table}{cols}{$col}{type};
+            my $primary  = $map{$table}{cols}{$col}{primary};
+            
             my ($field, $type, $ref) = _accessor_type($table, $col);
+
+            if ($primary or $type eq "id") {
+                croak "Two primary key columns detected for the '$table' "
+                    . "table: '$CLASS{$class}{id_col}' and '$col'"
+                    if $CLASS{$class}{id_col};
+            
+                $CLASS{$class}{accessors}{"id"} = {
+                    col => $col,
+                    type => "id",
+                    ref => ""
+                };
+                $CLASS{$class}{id_col} = $col;
+                next;
+            }
             
             $CLASS{$class}{accessors}{$field} = {
                 col  => $col,
@@ -346,66 +369,78 @@ sub _parse_tables {
                 ref  => $ref,
             };
 
-            $CLASS{$class}{id_col} = $col, next if $type eq 'id';
-
-            ## reverse-map the direct foreign keys
-            if ($type eq 'direct') {
+            ## reverse the foreign keys in the appropriate class
+            if ($type eq '1-to-1') {
                 my $ref_class = $TABLE_MAP{$ref};
                 $CLASS{$ref_class}{accessors}{$table} = {
                     col  => $field,
-                    type => 'indirect',
+                    type => '1-to-n',
                     ref  => $table
                 };
             }
 
+            $CLASS{$class}{order_by} = $col
+                unless exists $CLASS{$class}{order_by};
+            
             push @{ $CLASS{$class}{load_cols} }, $col
-                if $hr->{Type} !~ /blob|text/;
+                unless $col_type =~ /blob|text|bytea/;
                 
-            $CLASS{$class}{order_by} ||= $col;
         }
-        $q->finish;
-        
-    }
 
+    }
+    
+    
+    if ($SCHEMA_CACHE) {
+        $CACHE->{$signature}{CLASS}     = \%CLASS;
+        $CACHE->{$signature}{TABLE_MAP} = \%TABLE_MAP;
+        nstore $CACHE, $SCHEMA_CACHE;
+    }
+    
 }
 
 #####
 
 sub _accessor_type {
-    my ($table, $name) = @_;
+    my ($table, $col) = @_;
+
+    ## $name is the name of the accessor *method*
+    ## $ref is the name of the table being referred to, for foreign keys
 
     my $type = 'normal';
     my $ref  = '';
+    my $name = $col;
     
-    if ($name =~ s/_id$//) {
-        if ($name eq $table) {
-            $name = $type = 'id';
-        } elsif (PL_N($name) eq $table) {
-            $name = $type = 'id';
-        } elsif (exists $TABLE_MAP{$name}) {
-            $type = 'direct';
-            $ref  = $name;
-        } elsif ( exists $TABLE_MAP{ PL_N($name) } ) {
-            $type = 'direct';
-            $ref  = PL_N($name);
+    ## remove optional prefix of "tablename_" (accounting for singular/plural)
+    ## .. couldn't do this with one giant extended regex, because PL_N re-
+    ## invokes the regex engine.
+    
+    while ($col =~ /_/g) {
+        my $pre  = substr($col, 0, pos($col) - 1);
+        my $post = substr($col, pos $col);
+        
+        if ($table eq $pre or $table eq plural($pre)) {
+            $name = $col = $post;
+            last;
         }
-    } elsif ($name eq 'id') {
-        $type = 'id';
-    } elsif (exists $TABLE_MAP{$name}) {
-        $type = 'direct';
-        $ref  = $name;
-    } elsif (exists $TABLE_MAP{ PL_N($name) }) {
-        $type = 'direct';
-        $ref  = PL_N($name);
     }
-
-    ## $name is the name of the accessor METHOD
-    ## $ref  is the name of the table in a direct accessor
-
+    
+    $col =~ s/_id$//;
+    
+    if ( $col eq "id" ) {
+        $name = $type = 'id';
+    }
+    
+    if ( exists $TABLE_MAP{$col} or exists $TABLE_MAP{plural($col)} ) {
+        $name = $col;
+        $type = "1-to-1";
+        $ref  = exists $TABLE_MAP{$col} ? $col : plural($col);
+    }
+                
     return ($name, $type, $ref);
 }
 
 sub _table_to_package_name {
+    return join "" => map ucfirst, split /_/, lc shift;
     my $table = lc shift;
     $table =~ s/(?:^|_)(.)/uc $1/ge;
     return $table;
@@ -444,18 +479,15 @@ sub sql_query {
     return $sth;
 }
 
-sub sql_insertid {
-    return $DBH->{'mysql_insertid'};
-}
-
 sub sql_do {
-    my $sth = sql_query(@_) or return undef;
-
-    my @ret = $_[0] =~ /^\s*select/i 
+    my $sql = shift;    
+    my $sth = sql_query($sql, @_) or return undef;
+    
+    my @ret = $sql =~ /^\s*select/i
         ? $sth->fetchrow_array
         : (1);
+    
     $sth->finish;
-
     return wantarray ? @ret : $ret[0];
 }
 
@@ -493,19 +525,19 @@ following them.
 
 =head2 Introductory Example
 
-Suppose your database schema were as unweildy as the following SQL. The
+Suppose your database schema were as unweildy as the following MySQL. The
 incosistent naming, the plural table names and singular column names are not
 a problem for Class::Tables.
 
   create table departments (
-      id            int not null primary key auto_increment,
-      name          varchar(50) not null
+      id                int not null primary key auto_increment,
+      department_name   varchar(50) not null
   );
   create table employees (
-      employee_id   int not null primary key auto_increment,
-      name          varchar(50) not null,
-      salary        int not null,
-      department_id int not null
+      employee_id       int not null primary key auto_increment,
+      name              varchar(50) not null,
+      salary            int not null,
+      department_id     int not null
   );
 
 To use Class::Tables, you need to do no more than this:
@@ -546,7 +578,9 @@ singular and "employees" is plural (See L<Plural And Singular Nouns>).
 
 C<Departments> objects get a C<name> accessor/mutator method, and C<Employees>
 objects get C<name> and C<salary> accessor/mutator methods, referring to the
-respective columns in the database.
+respective columns in the database. Note that the C<department_> prefix is
+automatically removed from the C<department_name> column because the name of
+the table is C<departments>.
 
   $self->salary(int rand 100_000);
   print "Pass go, collect " . $self->salary . " dollars";
@@ -595,34 +629,50 @@ the name of the class/package.
 
 =item Primary Key
 
-All tables must have a primary key column in the database, which is an integer
-column set to C<AUTO_INCREMENT>. This column must be named either C<id>, the
-table name followed by an C<_id> suffix, or the singular form of the table
-name followed by an C<_id> suffix.
+All tables must have a integer single-column primary key. By default,
+Class::Tables will use the primary key from the table definition. If no
+column is explicitly listed as the primary key, it will try to find one by
+name: valid names are C<id> or the table name (plus or minus pluralization)
+followed by an C<_id> suffix.
 
-In our above example, the C<employees> table could have had a primary key
-column named C<employee_id>, C<employees_id>, or C<id>. The flexibility allows
-for reasonable choices whether you name your tables as singular or plural
-nouns. (See L<Plural And Singular Nouns>)
+In our above example, if the C<primary key> keyword was omitted from the
+C<employees> table definition, Class::Tables would have looked for columns
+named C<employee_id>, C<employees_id>, or C<id> as the primary key. The
+flexibility allows for reasonable choices whether you name your tables as
+singular or plural nouns. (See L<Plural And Singular Nouns>)
 
-For simplicity and transparency, the associated object accessor is always
-named C<id>, regardless of the underlying column name.
+Note: For simplicity and transparency, the associated object accessor is
+always named C<id>, regardless of the underlying column name.
+
+In MySQL, the primary key column must be set to C<AUTO_INCREMENT>.
+
+In SQLite, the primary key may be an auto increment column (in SQLite this is
+only possible if the column is the first column in the table and declared as
+C<integer primary key>) using the same naming conventions as above.
+Alternatively, you may omit an explicit primary key column and Class::Tables
+will use the hidden C<ROWID> column.
+
+In Postgres, the primary key column must be a C<serial primary key>. Using
+the hidden C<oid> column as primary key is not (yet) supported.
 
 =item Foreign Key Inflating
 
-If a column has the same name as another table, that column is treated as a
-foreign key reference to that table. Alternately, the column may be the
-singular form of the table name, and an optional C<_id> suffix may be added.
-The name of the accessor is the name of the column, minus the C<_id>.
+If a column has the same name as another table (plus or minus pluralization),
+that column is treated as a foreign key reference to that table. The column
+name may also have an optional C<_id> suffix and C<tablename_> prefix, where
+C<tablename> is the name of the current table (plus or minus pluralization).
+The name of the accessor is the name of the column, without the optional
+prefix and suffix.
 
 In our above example, the foreign key column relating each employee with a
-department could have been named C<departments>, C<departments_id>,
-C<department>, or C<department_id>. Again, the flexibility allows for a
-meaningful column name whether your table names are singular or plural. (See
+department could have been anything matching 
+C</^(employees?_)?(departments?)(_id)?$/>, with the accessor being named the
+value of $2 in that expression. Again, the flexibility allows for a meaningful
+column name whether your table names are singular or plural. (See
 L<Plural And Singular Nouns>).
 
 The foreign key relationship is also reversed as described in the example. The
-name of the accessor in the opposite direction is the name of the table. In
+name of the accessor in the opposite 1-to-1ion is the name of the table. In
 our example, this means that objects of the C<Departments> class get an
 accessor named C<employees>. For this reason, it is often convenient to name
 the tables as plural nouns.
@@ -641,7 +691,7 @@ on C<name>.
 
 =item Stringification
 
-If the table has a C<name> column, then any objects of that type will
+If the table has a C<name> accessor, then any objects of that type will
 stringify to the value of C<< $obj->name >>. Otherwise, the object will
 stringify to C<CLASS:ID>.
 
@@ -652,12 +702,40 @@ stringify to C<CLASS:ID>.
 
 =over
 
-=item C<< use Class::Tables $option >>
+=item C<< use Class::Tables %args >>
+
+Valid argument keys are:
+
+=over
+
+=item cascade
+
+Takes a boolean value indicating whether to perform cascading deletes. See
+C<delete> below for information on cascading deletes. If you need to change
+cascading delete behavior on the fly, localize C<$Class::Tables::CASCADE>.
+
+=item inflect
+
+Takes a boolean value indicating whether to use
+L<Lingua::EN::Inflect|Lingua::EN::Inflect> for plural & singular nouns. See
+L<Plural And Singular Nouns> for more information on noun pluralization.
+
+=item cache
+
+Takes a filename argument of a schema cache. This speeds up slow databases and
+large schemas. It uses L<Storable|Storable> to save the results of the schema
+mapping, and on each subsequent execution, uses the cache to keep from doing
+the mapping again. If your database's schema changes, simply empty the cache
+file to force a re-mapping.
+
+You can omit this arg or pass a false value to disable this feature.
+
+=back
+
+The default behavior is:
+
+  use Class::Tables cascade => 1, inflect => 1, cache => undef;
 
-C<$option> may be either C<'no_cascade'> or C<'cascade'> to disable and
-enable cascading deletes, respectively. See C<delete> below. The default
-is to enable cascading deletes. If you need to change cascading delete
-behavior on the fly, set C<$Class::Tables::CASCADE>.
 
 =item C<< Class::Tables->dbh($dbh) >>
 
@@ -724,7 +802,11 @@ appropriate class, which means you can also pass additional constraints:
 
   my @volunteers = $marketing->employees( salary => 0 );
   ## same as Employees->search( department => $marketing, salary => 0 );
-  
+
+For I<all> columns in the table C<tablename>, the column name will have any
+C<tablename_> prefix removed in the name of the accessor (plus or minus
+pluralization). So in the Employees table, the column name is effectively
+treated with C<s/^employees?_//> before consideration.
 
 =item C<< $obj->field($field) >> and C<< $obj->field($field, $new_val) >>
 
@@ -732,6 +814,16 @@ This is an alternative syntax to accessors/mutators. When C<$field> is the
 name of a valid accessor/mutator method for the object, this is equivalent to
 saying C<< $obj->$field >> and C<< $obj->$field($new_val) >>.
 
+=item C<< $obj->field >>
+
+With no arguments, the C<field> accessor returns a list of all the data
+accessors for that type of object. It's the same idea as CGI's C<param> method
+with no arguments.
+
+  for my $accessor ($obj->field) {
+      printf "$accessor : %s\n", scalar $obj->$accessor;
+  }
+
 =item C<< $obj->dump >>
 
 Returns a hashref containing the object's attribute data. It recursively
@@ -805,8 +897,8 @@ equivalent to the following:
 So for foreign key attributes, you may pass an actual object or an ID:
 
   ## both are ok:
-  Employee->new( department => $marketing );
-  Employee->new( department => 10 );
+  my $e = Employees->new( department => $marketing );
+  my $e = Employees->new( department => 10 );
 
 =item C<< Class->search( field1 => $value1, field2 => $value2, ... ) >>
 
@@ -816,6 +908,9 @@ objects matched). In scalar context returns only the first object returned
 by the query (or C<undef> if no objects matched). The scalar context SQL query
 is slightly optimized.
 
+C<field1>, C<field2>, etc, must be names of the I<accessors>, and not the
+underlying column.
+
 As usual, for foreign key attributes, you may pass an actual object or an ID.
 If no arguments are passed to C<search>, every object in the class is
 returned.
@@ -875,31 +970,79 @@ attributes with the objects:
       @_ ? $foo{ $self->id } = shift
          : $foo{ $self->id };
   }
+  sub DESTROY {
+      my $self = shift;
+      delete $foo{ $self->id };
+      $self->SUPER::DESTROY;
+  }
+
+=head2 Subclassing/wrapping Class::Tables
+
+You may find it necessary to subclass Class::Tables. One example I can think
+of is to implement a security wrapper. This is pretty simple. You can simply
+add wrappers to the C<new>, C<fetch>, and C<search> methods as necessary. For
+a really hairbrained example, say that we want to restrict certain users of
+our application to objects with I<odd-numbered IDs only>:
+
+  package MySubclass;
+  use base 'Class::Tables';
+  
+  sub fetch {
+      my ($pkg, $id) = @_;
+
+      warn("Odd numbers only!") and return
+          unless privileged( get_current_user() ) or $id % 2;
+
+      $pkg->SUPER::fetch($id);
+  }
+
+  sub search {
+      my $pkg = shift;
+      my @results = $pkg->SUPER::search(@_);
+
+      ## something similarly appropriate if the current user is
+      ## unprivileged, perhaps along the lines of:
+      ##     grep { $_->id % 2 } @results
+      ## or raising warnings, etc.
+  }
+
+To use this subclass, simply change these two lines in the application code
+and the persistence classes will be created beneath C<MySubclass> instead of
+beneath Class::Tables:
+
+  ## use Class::Tables;
+  ## Class::Tables->dbh($dbh);
+  use MySubclass;
+  MySubclass->dbh($dbh);
+
+  ## this will now raise a warning if privileges not met
+  my $obj = Employees->fetch(10);
+
+
 
 =head2 Plural And Singular Nouns
 
 Class::Tables makes strong use of L<Lingua::EN::Inflect|Lingua::EN::Inflect>
 to convert between singular and plural, in an effort to make accessor names
 more meaningful and allow a wide range of column-naming schemes. So when this
-documentation talks about plural and singular nouns, it does not just mean
+documentation says "plus or minus pluralization", it does not just consider
 "adding an S at the end." You zoologists may have a C<mice> table with a
 corresponding primary/foreign key named C<mouse_id>! Goose to geese, child to
 children, etc. The only limitations are what
 L<Lingua::EN::Inflect|Lingua::EN::Inflect> doesn't know about.
 
 I recommend naming tables with a plural noun and foreign key columns with a
-singular noun (optionally with the "_id" suffix). This combination makes the
+singular noun (optionally with the C<_id> suffix). This combination makes the
 accessor names much more meaningful, and is (to my knowledge) the most common
 relational naming convention.
 
 If L<Lingua::EN::Inflect|Lingua::EN::Inflect> is not available on your system,
-Class::Tables will still work fine, but without the distinction between
-plurals and singulars. Thus a primary key column can be named only C<id> or
-the name of the table with an C<_id> suffix. Similar statments are true for
-foreign key columns, etc.
+Class::Tables will still work fine, but using a very naive singular-to-plural
+conversion algorithm ("adding an S at the end").
 
-You can manually disable the pluralization by setting
-C<$Class::Tables::PLURALIZE> to a false value before you generate the classes.
+You can manually disable using L<Lingua::EN::Inflect|Lingua::EN::Inflect> with
+
+  use Class::Tables inflect => 0;
 
 =head1 CAVEATS
 
@@ -907,8 +1050,11 @@ C<$Class::Tables::PLURALIZE> to a false value before you generate the classes.
 
 =item *
 
-So far, the table parsing code only works with MySQL. Same with getting the
-ID of the last inserted object. Testers/patchers for other RDBMSs welcome!
+Supported database drivers are MySQL, SQLite, and Postgres. With SQLite, you
+can only get an auto-incremented ID column if it is the first column in the
+table, and if it is declared as C<integer primary key> (or you can just use
+the hidden C<rowid> column). With Postgres, only C<serial primary key>s are
+supported (usign the C<oid> column as primary key is not yet supported).
 
 =item *
 
@@ -919,6 +1065,12 @@ Pluralization code is only for English at the moment, sorry.
 All modifications to objects are instantaneous -- no asynchronous updates
 and/or rollbacks (yet?)
 
+=item *
+
+Only one database handle is used at a time. Calling C<< Class::Tables->dbh >>
+a second time will produce undefined results. The parameters used in the
+C<use Class::Tables> line are global.
+
 =back
 
 =head1 AUTHOR
@@ -928,7 +1080,6 @@ free to contact me with comments, questions, patches, or whatever.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003 Mike Rosulek. All rights reserved. This module is free 
+Copyright (c) 2004 Mike Rosulek. All rights reserved. This module is free 
 software; you can redistribute it and/or modify it under the same terms as Perl 
 itself.
-
